@@ -3,12 +3,12 @@
  *
  * This file is part of msmtp, an SMTP client.
  *
- * Copyright (C) 2000, 2003, 2004, 2005, 2006
+ * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007, 2008
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -17,8 +17,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software Foundation,
- *   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -29,10 +28,11 @@
 #include <limits.h>
 #include <stdio.h>
 #include <string.h>
+#include <strings.h>
 #include <ctype.h>
 #include <errno.h>
-extern int errno;
 
+#include "c-ctype.h"
 #include "gettext.h"
 #include "xalloc.h"
 #include "xvasprintf.h"
@@ -79,8 +79,11 @@ account_t *account_new(const char *conffile, const char *id)
     a->tls_key_file = NULL;
     a->tls_cert_file = NULL;
     a->tls_trust_file = NULL;
+    a->tls_crl_file = NULL;
     a->tls_nocertcheck = 0;
     a->tls_force_sslv3 = 0;
+    a->tls_min_dh_prime_bits = -1;
+    a->tls_priorities = NULL;
     a->logfile = NULL;
     a->syslog = NULL;
     return a;
@@ -125,8 +128,13 @@ account_t *account_copy(account_t *acc)
 	    acc->tls_cert_file ? xstrdup(acc->tls_cert_file) : NULL;
 	a->tls_trust_file = 
 	    acc->tls_trust_file ? xstrdup(acc->tls_trust_file) : NULL;
+	a->tls_crl_file = 
+	    acc->tls_crl_file ? xstrdup(acc->tls_crl_file) : NULL;
 	a->tls_nocertcheck = acc->tls_nocertcheck;
 	a->tls_force_sslv3 = acc->tls_force_sslv3;
+	a->tls_min_dh_prime_bits = acc->tls_min_dh_prime_bits;
+	a->tls_priorities = 
+	    acc->tls_priorities ? xstrdup(acc->tls_priorities) : NULL;
 	a->logfile = acc->logfile ? xstrdup(acc->logfile) : NULL;
 	a->syslog = acc->syslog ? xstrdup(acc->syslog) : NULL;
     }
@@ -158,6 +166,8 @@ void account_free(void *a)
 	free(p->tls_key_file);
 	free(p->tls_cert_file);
 	free(p->tls_trust_file);
+	free(p->tls_crl_file);
+	free(p->tls_priorities);
 	free(p->dsn_return);
 	free(p->dsn_notify);
 	free(p->logfile);
@@ -208,7 +218,7 @@ account_t *find_account_by_envelope_from(list_t *acc_list, const char *from)
     {
 	acc_list = acc_list->next;
 	acc_from = ((account_t *)(acc_list->data))->from;
-	if (acc_from && strcmp(from, acc_from) == 0)
+	if (acc_from && strcasecmp(from, acc_from) == 0)
 	{
 	    a = acc_list->data;
 	    break;
@@ -288,7 +298,7 @@ int check_auth_arg(char *arg)
     	l = strlen(arg);
 	for (i = 0; i < l; i++)
 	{
-	    arg[i] = toupper((unsigned char)arg[i]);
+	    arg[i] = c_toupper((unsigned char)arg[i]);
 	}
 	return 0;
     }
@@ -342,7 +352,7 @@ int check_dsn_notify_arg(char *arg)
     l = strlen(arg);
     for (i = 0; i < l; i++)
     {
-	arg[i] = toupper((unsigned char)arg[i]);
+	arg[i] = c_toupper((unsigned char)arg[i]);
     }
     return 0;
 }
@@ -486,14 +496,30 @@ void override_account(account_t *acc1, account_t *acc2)
 	acc1->tls_trust_file = 
 	    acc2->tls_trust_file ? xstrdup(acc2->tls_trust_file) : NULL;
     }
-    if (acc2->mask & ACC_TLS_FORCE_SSLV3)
+    if (acc2->mask & ACC_TLS_CRL_FILE)
     {
-	acc1->tls_force_sslv3 = acc2->tls_force_sslv3;
+	free(acc1->tls_crl_file);
+	acc1->tls_crl_file = 
+	    acc2->tls_crl_file ? xstrdup(acc2->tls_crl_file) : NULL;
     }
     if (acc2->mask & ACC_TLS_NOCERTCHECK)
     {
 	acc1->tls_nocertcheck = acc2->tls_nocertcheck;
     }
+    if (acc2->mask & ACC_TLS_FORCE_SSLV3)
+    {
+	acc1->tls_force_sslv3 = acc2->tls_force_sslv3;
+    }
+    if (acc2->mask & ACC_TLS_MIN_DH_PRIME_BITS)
+    {
+	acc1->tls_min_dh_prime_bits = acc2->tls_min_dh_prime_bits;
+    }
+    if (acc2->mask & ACC_TLS_PRIORITIES)
+    {
+	free(acc1->tls_priorities);
+	acc1->tls_priorities = acc2->tls_priorities 
+	    ? xstrdup(acc2->tls_priorities) : NULL;
+    }
     if (acc2->mask & ACC_DSN_RETURN)
     {
 	free(acc1->dsn_return);
@@ -561,6 +587,18 @@ int check_account(account_t *acc, int sendmail_mode, char **errstr)
 		_("cannot use tls_trust_file with tls_certcheck turned off"));
 	return CONF_ESYNTAX;
     }
+    if (acc->tls && !acc->tls_trust_file && !acc->tls_nocertcheck)
+    {
+	*errstr = xasprintf(
+		_("tls requires either tls_trust_file (highly recommended) or "
+		    "a disabled tls_certcheck"));
+	return CONF_ESYNTAX;
+    }
+    if (acc->tls_crl_file && !acc->tls_trust_file)
+    {
+	*errstr = xasprintf(_("tls_crl_file requires tls_trust_file"));
+      	return CONF_ESYNTAX;
+    }
 
     return CONF_EOK;
 }
@@ -1159,6 +1197,19 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 		acc->tls_trust_file = expand_tilde(arg);
 	    }
 	}
+	else if (strcmp(cmd, "tls_crl_file") == 0)
+	{
+	    acc->mask |= ACC_TLS_CRL_FILE;
+	    free(acc->tls_crl_file);
+	    if (*arg == '\0')
+	    {
+		acc->tls_crl_file = NULL;
+	    }
+	    else
+	    {
+		acc->tls_crl_file = expand_tilde(arg);
+	    }
+	}
 	else if (strcmp(cmd, "tls_certcheck") == 0)
 	{
 	    acc->mask |= ACC_TLS_NOCERTCHECK;
@@ -1199,6 +1250,39 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 		break;
 	    }
 	}
+	else if (strcmp(cmd, "tls_min_dh_prime_bits") == 0)
+	{
+	    acc->mask |= ACC_TLS_MIN_DH_PRIME_BITS;
+	    if (*arg == '\0')
+	    {
+		acc->tls_min_dh_prime_bits = -1;
+	    }
+	    else
+	    {
+		acc->tls_min_dh_prime_bits = get_pos_int(arg);
+		if (acc->tls_min_dh_prime_bits < 1)
+		{
+		    *errstr = xasprintf(
+			    _("line %d: invalid argument %s for command %s"),
+			    line, arg, cmd);
+		    e = CONF_ESYNTAX;
+		    break;
+		}
+	    }
+	}
+	else if (strcmp(cmd, "tls_priorities") == 0)
+	{
+	    acc->mask |= ACC_TLS_PRIORITIES;
+	    free(acc->tls_priorities);
+	    if (*arg == '\0')
+	    {
+		acc->tls_priorities = NULL;
+	    }
+	    else
+	    {
+		acc->tls_priorities = xstrdup(arg);
+	    }
+	}
 	else if (strcmp(cmd, "dsn_return") == 0)
 	{
 	    acc->mask |= ACC_DSN_RETURN;
