@@ -3,12 +3,12 @@
  * 
  * This file is part of msmtp, an SMTP client.
  *
- * Copyright (C) 2000, 2003, 2004, 2005, 2006
+ * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -17,8 +17,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software Foundation,
- *   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -32,7 +31,6 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
-extern int errno;
 #include <sys/time.h>
 #include <sys/types.h>
 #ifdef W32_NATIVE
@@ -40,13 +38,13 @@ extern int errno;
 # include <winsock2.h>
 # include <ws2tcpip.h>
 #else
+# include <sys/socket.h>
+# include <arpa/inet.h>
+# include <netdb.h>
 # ifndef HAVE_GETADDRINFO
 #  include <netinet/in.h>
    extern int h_errno;
 # endif
-# include <sys/socket.h>
-# include <arpa/inet.h>
-# include <netdb.h>
 # ifndef NI_MAXHOST
 #  define NI_MAXHOST 1025
 # endif
@@ -60,9 +58,46 @@ extern int errno;
 #include "xalloc.h"
 #include "xvasprintf.h"
 
+#include "readbuf.h"
 #include "net.h"
 
 
+/*
+ * hstrerror()
+ *
+ * This function is only used on systems that 
+ * 1. lack getaddrinfo(), so that gethostbyname() must be used instead, and
+ * 2. do not provide hstrerror() themselves.
+ * The messages are identical to the ones in wsa_strerror() 
+ * below so that no additional strings have to be translated. 
+ */
+
+#ifndef HAVE_GETADDRINFO
+#ifndef HAVE_HSTRERROR
+const char *hstrerror(int e)
+{
+    switch (e)
+    {
+	case HOST_NOT_FOUND:
+    	    return _("host not found (authoritative)");
+
+	case TRY_AGAIN:
+    	    return _("host not found (nonauthoritative) or server failure");
+
+	case NO_RECOVERY:
+    	    return _("nonrecoverable error");
+	
+	case NO_DATA:
+    	    return _("valid name, but no data record of requested type");
+	
+	default:        /* should never happen */
+    	    return _("unknown error");
+    }
+}
+#endif
+#endif
+
+
 /*
  * [Windows only] wsa_strerror() 
  *
@@ -316,14 +351,23 @@ int net_connect(int fd, const struct sockaddr *serv_addr, socklen_t addrlen,
 
 void net_set_io_timeout(int socket, int seconds)
 {
+#ifdef W32_NATIVE
     /* On old Windows systems (older than Windows 2000), these timeouts are 
      * broken, see http://msdn.microsoft.com/library/default.asp?url=/library/
      * en-us/winsock/winsock/setsockopt_2.asp
      * We activate these timeouts only for Windows systems that also have
      * getaddrinfo(), which means XP or newer, to work around this problem. */
-#if defined W32_NATIVE && !defined HAVE_GETADDRINFO
-    /* do nothing */
-#else
+# ifdef HAVE_GETADDRINFO
+    DWORD milliseconds;
+
+    if (seconds > 0)
+    {
+	milliseconds = seconds * 1000;
+	(void)setsockopt(socket, SOL_SOCKET, SO_RCVTIMEO, &milliseconds, sizeof(int));
+	(void)setsockopt(socket, SOL_SOCKET, SO_SNDTIMEO, &milliseconds, sizeof(int));
+    }
+# endif
+#else /* UNIX or DJGPP */
     struct timeval tv;
     
     if (seconds > 0)
@@ -613,25 +657,13 @@ int net_open_socket(const char *hostname, int port, int timeout, int *ret_fd,
 }
 
 
-/*
- * net_readbuf_init()
- *
- * see net.h
- */
-
-void net_readbuf_init(net_readbuf_t *readbuf)
-{
-    readbuf->count = 0;
-}
-
-
 /*
  * net_readbuf_read()
  *
  * Wraps read() to provide buffering for net_gets().
  */
 
-int net_readbuf_read(int fd, net_readbuf_t *readbuf, char *ptr, 
+int net_readbuf_read(int fd, readbuf_t *readbuf, char *ptr, 
 	char **errstr)
 {
 #ifdef W32_NATIVE
@@ -709,7 +741,7 @@ int net_readbuf_read(int fd, net_readbuf_t *readbuf, char *ptr,
  * see net.h
  */
 
-int net_gets(int fd, net_readbuf_t *readbuf, 
+int net_gets(int fd, readbuf_t *readbuf, 
 	char *str, size_t size, size_t *len, char **errstr)
 {
     char c;
